"""Utility functions for the upload cli commands."""

import logging
from pathlib import Path
from typing import Optional

import click

from cg.constants import Pipeline
from cg.constants.constants import MAX_ITEMS_TO_RETRIEVE
from cg.store import Store
from cg.store.models import Family

LOG = logging.getLogger(__name__)


def suggest_cases_to_upload(status_db: Store, pipeline: Optional[Pipeline] = None) -> None:
    """Print a list of suggested cases to upload."""
    LOG.warning("Provide a case, suggestions:")
    records = status_db.analyses_to_upload(pipeline=pipeline)[:MAX_ITEMS_TO_RETRIEVE]
    for case_obj in records:
        click.echo(case_obj)



"""Handler to delete data objects"""

from typing import List
from cg.store.models import Flowcell, Family, Sample
from cg.store.api.base import BaseHandler


class DeleteDataHandler(BaseHandler):
    """Contains methods to delete business data model instances"""

    def delete_flow_cell(self, flow_cell_name: str) -> None:
        """Delete flow cell."""
        flow_cell: Flowcell = self.Flowcell.query.filter(Flowcell.name == flow_cell_name).first()

        if flow_cell:
            flow_cell.delete()
            flow_cell.flush()
            self.commit()

    def delete_relationships_sample(self, sample: Sample) -> None:
        """Delete relationships between all cases and the provided sample."""
        if sample and sample.links:
            for case_sample in sample.links:
                case_sample.delete()
            self.commit()

    def delete_cases_without_samples(self, case_ids: List[str]) -> None:
        """Delete any cases specified in case_ids without samples."""
        for case_id in case_ids:
            case: Family = self.Family.query.filter(Family.internal_id == case_id).first()
            if case and not case.links:
                case.delete()
        self.commit()



from enum import Enum
from typing import List, Optional, Callable

from sqlalchemy.orm import Query

from cg.store.models import BedVersion


def get_bed_version_by_short_name(
    bed_versions: Query, bed_version_short_name: str, **kwargs
) -> Query:
    """Return beds by short name."""
    return bed_versions.filter(BedVersion.shortname == bed_version_short_name)


class BedVersionFilters(Enum):
    """Define BED version filter functions."""

    get_bed_version_by_short_name: Callable = get_bed_version_by_short_name


def apply_bed_version_filter(
    bed_versions: Query,
    functions: List[Callable],
    bed_version_short_name: Optional[str] = None,
) -> Query:
    """Apply filtering functions and return filtered results."""
    for function in functions:
        bed_versions: Query = function(
            bed_versions=bed_versions,
            bed_version_short_name=bed_version_short_name,
        )
    return bed_versions
